/*
 * Sangoma Media Processing Library
 *
 * Moises Silva <moy@sangoma.com>
 *
 * Copyright (C) 2011 Sangoma Technologies
 *
 * Redistribution and use is permitted only in binary forms without
 * modification and provided that the following conditions are met:
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contributors:
 *
 */

#ifndef _SNGMP_CORE_H_
#define _SNGMP_CORE_H_

#include "sngmp_declare.h"

#include <inttypes.h>

/* for sockaddr_storage */
#include <netinet/in.h>


SNGMP_BEGIN_EXTERN_C 

/*! \brief Crash policy
 *         Useful for debugging only, default policy is never, if you wish to crash on asserts 
 *         then use sngmp_core_set_crash_policy 
 */
typedef enum {
	SNGMP_CRASH_NEVER = 0,
	SNGMP_CRASH_ON_ASSERT
} sngmp_crash_policy_t;

/*! \brief Crash policy for the library. Use sngmp_core_set_crash_policy to set the policy */
SNGMP_DECLARE_DATA extern sngmp_crash_policy_t sngmp_global_crash_policy;

/*! \brief Set the library crash policy */
SNGMP_DECLARE(sngmp_result_t) sngmp_core_set_crash_policy(sngmp_crash_policy_t policy);

/*! \brief Custom memory handler hooks. Not recommended to use unless you need memory allocation customizations */
typedef void *(*sngmp_malloc_func_t)(void *pool, sngmp_size_t len);
typedef void *(*sngmp_calloc_func_t)(void *pool, sngmp_size_t elements, sngmp_size_t len);
typedef void *(*sngmp_realloc_func_t)(void *pool, void *buff, sngmp_size_t len);
typedef void (*sngmp_free_func_t)(void *pool, void *ptr);
typedef struct sngmp_global_memory_handler_s {
	void *pool;
	sngmp_malloc_func_t malloc;
	sngmp_calloc_func_t calloc;
	sngmp_realloc_func_t realloc;
	sngmp_free_func_t free;
} sngmp_global_memory_handler_t;

/*! \brief The memory handler. 
     Do not use directly this variable, use the memory macros and sngmp_global_set_memory_handler to override */	
SNGMP_DECLARE_DATA extern sngmp_global_memory_handler_t sngmp_global_memory_handler;

/*! \brief Abstract network address (functions sngmp_sockaddr_xxx operating on this type are not thread-safe) */
struct sngmp_sockaddr_s {
	/*! this will be either a sockaddr_in or sockaddr_in6 */
	struct sockaddr_storage ss;
	/*! Length of the socket storage */
	socklen_t len;
	/*! Buffer to stringify the address */
	char ipbuff[255];
};

/*! IP version 4 */
typedef uint32_t sngmp_ipv4_t;

/*! \brief Whether to enable RTCP or not */
#define SNGMP_MEDIA_RTCP_ENABLE (1 << 0)

/*! \brief Media description (simplified SDP-like data structure) */
struct sngmp_media_description_s {
	/*! Session name */
	char session_name[255];
	/*! Audio, Video, etc */
	sngmp_media_type_t type;
	/*! Payload number to use */
	uint8_t payload;
	/*! IANA name for this type of media */
	char iananame[255];
	/*! Samples per second */
	uint32_t samples_per_second;
	/*! Bits per second (bit rate) */
	uint32_t bits_per_second;
	/*! 20ms, 30ms, 40ms etc */
	uint32_t ptime;
	/*! Media flags */
	sngmp_flags_t flags;
};

/*! \brief Set the library memory handler (default is used otherwise) */
SNGMP_DECLARE(sngmp_result_t) sngmp_core_set_memory_handler(sngmp_global_memory_handler_t *handler);

/*! \brief Copy socket address */
SNGMP_DECLARE(sngmp_result_t) sngmp_sockaddr_copy(sngmp_sockaddr_t *dst, sngmp_sockaddr_t *src);

/*! \brief Initialize sockaddr using the given sockaddr_in structure */
SNGMP_DECLARE(sngmp_result_t) sngmp_sockaddr_init_from_sin(sngmp_sockaddr_t *addr, struct sockaddr_in *sinaddr);

/*! \brief Initialize the address from a string/port pair
 *  \param addr Pointer to the abstract address storage
 *  \param ipaddr The string IP address (either IPv4 or IPv6)
 *  \param port the IP port (host byte order)
 */
SNGMP_DECLARE(sngmp_result_t) sngmp_sockaddr_init(sngmp_sockaddr_t *addr, const char *ipaddr, sngmp_ip_port_t port);

/*! \brief Check if a given address is null */
SNGMP_DECLARE(sngmp_bool_t) sngmp_sockaddr_isnull(sngmp_sockaddr_t *addr);

/*! \brief Get the IP port from the given address */
SNGMP_DECLARE(sngmp_ip_port_t) sngmp_sockaddr_get_port(sngmp_sockaddr_t *addr);

/*! \brief Get the IPv4 from the the given address */
SNGMP_DECLARE(sngmp_ipv4_t) sngmp_sockaddr_get_ipv4(sngmp_sockaddr_t *addr);

/*! \brief Get the IPv4 string representation from the the given address */
SNGMP_DECLARE(const char *) sngmp_sockaddr_get_ipv4_str(sngmp_sockaddr_t *addr);

SNGMP_END_EXTERN_C

#endif /* endif defined _SNGMP_CORE_H_ */

