/*
 * Sangoma Media Processing Library
 *
 * Moises Silva <moy@sangoma.com>
 *
 * Copyright (C) 2011 Sangoma Technologies
 *
 * Redistribution and use is permitted only in binary forms without
 * modification and provided that the following conditions are met:
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contributors:
 *
 */


/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*\

    Block diagram:

           +------------------+  +---------+  +---------+  +------------------+
HOST<->|   | VOCALLO HOST A   |  |  VOC    |->|  VOC    |  | VOCALLO HOST B   |   |<->HOST
 A     |   +----------------+ |  |  TERM A |  |  TERM B |  | +----------------+   |    B
       |   | RTP SESSION A  | |  |         |  |         |  | | RTP SESSION B  |   |
       |   +--------------+ | |  |  G.711  |  |  G.711  |  | | +--------------+   |
       |<->| RTP MEMBER A |<|-|->| (U-LAW) |<-| (A-LAW) |<-|-|>| RTP MEMBER B |<->|
           +--------------+-+-+  +---------+  +---------+  +-+-+--------------+

|----------------- SIDE A ------------------||----------------- SIDE B ------------------|

\*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

#ifndef _SNGMP_RTP_SESSION_H_
#define _SNGMP_RTP_SESSION_H_

#include "sngmp_declare.h"
#include "sngmp_core.h"

SNGMP_BEGIN_EXTERN_C

/*! \brief RTP session */
typedef struct sngmp_rtp_session_s sngmp_rtp_session_t;
typedef struct sngmp_rtp_bridge_s sngmp_rtp_bridge_t;

/*! \brief Create a new RTP session 
 *         If both remote_sockaddr and description are given this function does
 *         an implicit call to sngmp_rtp_session_activate()
 *  \param session The session created
 *  \param remote_sockaddr The remote socket address to send media to. Optional.
 *  \param media The type of media to send and receive. Optional.
 *  \param mod The media processing module to use. Optional.
 */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_session_create(sngmp_rtp_session_t **session, 
						       sngmp_sockaddr_t *remote_sockaddr,
						       sngmp_media_description_t *media,
						       sngmp_module_t *mod);

/*! \brief Update the base information for an RTP session
 *  \param session The session created
 *  \param The remote socket address to send media. Optional.
 *  \param The type of media to send and receive. Optional.
 */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_session_update_info(sngmp_rtp_session_t *session,
							    sngmp_sockaddr_t *remote_sockaddr,
							    sngmp_media_description_t *description);

/*! \brief Starts the RTP session streaming */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_session_start_streaming(sngmp_rtp_session_t *session);

/*! \brief Stop the RTP session streaming */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_session_stop_streaming(sngmp_rtp_session_t *session);

/*! \brief Retrieve a copy of the socket address of the media processor associated with the session */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_session_get_sockaddr(sngmp_rtp_session_t *session, sngmp_sockaddr_t *sockaddr);

/*! \brief Retrieve the the media processor associated with the session */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_session_get_module(sngmp_rtp_session_t *session, sngmp_module_t **mod);

/*! \brief Bridges two RTP sessions 
 *  \note Be aware you can only bridge sessions that were created using the 
 *        same media processing module (last parameter of sngmp_rtp_session_create)
 */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_bridge_create(sngmp_rtp_bridge_t **bridge,
						      sngmp_rtp_session_t *sess_a,
						      sngmp_rtp_session_t *sess_b);

/*! \brief Destroy the RTP bridge */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_bridge_destroy(sngmp_rtp_bridge_t **bridge);

/*! \brief Destroy the RTP session */
SNGMP_DECLARE(sngmp_result_t) sngmp_rtp_session_destroy(sngmp_rtp_session_t **session);

SNGMP_END_EXTERN_C

#endif /* endif defined _SNGMP_RTP_SESSION_H_ */

